﻿<%@ WebHandler Language="C#" Debug="true" Class="StyleConfigLoader" %>

using System;
using System.Web;
using System.Linq;
using System.Collections.Generic;

public class StyleConfigLoader : IHttpHandler
{
    //add in other tags you would like the styles/classes to display for.
    private List<string> eType = new List<string>() { "P", "DIV", "SPAN" };
    //excluded selectors so we don't include them in our list to display
    private List<string> excludeSelectors = new List<string> { "h1", "h2", "h3", "h4", "h5", "h6", ".h1", ".h2", ".h3", ".h4", ".h5", ".h6", "p", "strong", "bold", "body" };

    private bool insertAsClass = true;

    public void ProcessRequest(HttpContext context)
    {
        context.Response.ContentType = "text/javascript";
        context.Response.Write(GenerateStyleConfig(context));
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

    /// <summary>
    /// Gets the styleconfig data to output to a friendly json file that can be read by the aloha editor in place of the styleconfig.js file.
    /// NOTE: you must add <add assembly="System.Runtime, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a" /> to compilation assembilies
    /// </summary>
    /// <param name="context"></param>
    /// <returns>output of json object</returns>
    private string GenerateStyleConfig(HttpContext context)
    {
        try
        {
            Ektron.Cms.Framework.Organization.FolderManager fManager = new Ektron.Cms.Framework.Organization.FolderManager();
            if (fManager.UserId > 0)
            {
                //check for a valid folder id
                if (context.Request.QueryString["fid"] != null)
                {
                    if (!string.IsNullOrEmpty(context.Request.QueryString["showstyle"]))
                    {
                        insertAsClass = false;
                    }
                    //read folder id from querystring
                    var fid = long.Parse(context.Request.QueryString["fid"]);
                    //see if we've already seen this folder and cache it.
                    string cacheKey = "StyleConfigJS" + insertAsClass + fid;
                    if (context.Cache[cacheKey] != null)
                    {
                        //return from cache
                        return (string)context.Cache[cacheKey];
                    }
                    else
                    {
                        //get the current folder's css specified

                        var folderData = fManager.GetItem(fid);
                        string ss = "";
                        //if no css is specified, check if it is inheriting
                        if (String.IsNullOrEmpty(folderData.StyleSheet))
                        {
                            while (folderData.ParentId != 0)
                            {
                                ss = fManager.GetItem(folderData.ParentId).StyleSheet;
                                if (ss != "")
                                {
                                    folderData.ParentId = 0;
                                    break;
                                }
                            }
                            if (ss == "")
                            {
                                ss = fManager.GetItem(folderData.ParentId).StyleSheet;
                            }
                        }
                        else
                        {
                            ss = folderData.StyleSheet;
                        }
                        if (!string.IsNullOrEmpty(ss))
                        {
                            //new parser object from the imported dll.
                            ExCSS.Parser parser = new ExCSS.Parser();
                            //load the stylesheet at the path
                            ExCSS.StyleSheet stylesheet = parser.Parse(System.IO.File.ReadAllText(context.Server.MapPath("/" + ss)));
                            //build a string for the styleconfig
                            System.Text.StringBuilder sb = new System.Text.StringBuilder();
                            System.Text.StringBuilder rules = new System.Text.StringBuilder();

                            //build rules
                            foreach (var rule in stylesheet.StyleRules)
                            {
                                foreach (string selector in rule.Value.ToString().Split(','))
                                {
                                    //make sure we want to inlude the selector
                                    if (ValidSelector(selector))
                                    {
                                        var selections = selector.Replace("\"", "");
                                        //duplicate selections for multiple selections per line that are comma separated.
                                        foreach (string sel in selections.Split(','))
                                        {
                                            //make sure we are looking at a class and not an ID. Also exclude chained css
                                            if (sel.Contains(".") && !sel.Contains(">"))
                                            {
                                                string selection = sel.Trim();
                                                if (sel.Contains(" "))
                                                {
                                                    selection = sel.Substring(0, sel.IndexOf(" "));
                                                }
                                                sb.Append(String.Format("\"{0}\":", selection.Replace(".", "")) + System.Environment.NewLine + "{" + System.Environment.NewLine);
                                                //build rules slightly differently to apply as a class or a style
                                                if (insertAsClass)
                                                {
                                                    sb.Append("\"type\": \"class\"," + System.Environment.NewLine);
                                                    sb.Append("\"value\": \"" + selection.Replace(".", "") + "\"," + Environment.NewLine);
                                                }
                                                else
                                                {
                                                    sb.Append("\"type\": \"style\"," + System.Environment.NewLine);
                                                    sb.Append("\"value\": \"" + rule.Declarations.ToString() + "\"," + Environment.NewLine);
                                                }
                                                sb.Append("\"workareaBox\": \"" + rule.Declarations.ToString() + "\"" + System.Environment.NewLine);
                                                sb.Append("}," + System.Environment.NewLine);
                                            }
                                        }
                                    }
                                }
                            }
                            rules.Append("{" + Environment.NewLine);
                            //loop through the selector display types to duplicate the list for each
                            foreach (string item in eType)
                            {
                                rules.Append("\"" + item + "\": {" + Environment.NewLine);
                                rules.Append("\"styles\"" + ": { " + System.Environment.NewLine);
                                rules.Append(sb.ToString());
                                rules = rules.Remove(rules.ToString().LastIndexOf(','), 1);
                                rules.Append("}" + Environment.NewLine + "}," + System.Environment.NewLine);
                            }

                            if (rules.Length > 1)
                            {
                                rules = rules.Remove(rules.ToString().LastIndexOf(','), 1);
                            }

                            var retVal = rules.Append(Environment.NewLine + "}").ToString();
                            context.Cache.Insert(cacheKey, retVal, null, DateTime.Now.AddHours(6), System.Web.Caching.Cache.NoSlidingExpiration);
                            return retVal;
                        }
                    }
                }
            }
        }
        catch { }
        //if a failure happens at any point, just load the styleconfig.js file
        return System.IO.File.ReadAllText(context.Server.MapPath("/Workarea/FrameworkUI/js/Ektron/Controls/EktronUI/Editor/Aloha/plugins/ektron/advancedinspector/lib/styleconfig.js"));
    }

    private bool ValidSelector(string selector)
    {
        if (excludeSelectors.IndexOf(selector) == -1)
        {
            return true;
        }
        return false;
    }

    private string BuildAttributes(string elementJS, ExCSS.StyleDeclaration values)
    {
        System.Text.StringBuilder sb = new System.Text.StringBuilder();
        if (values.Count > 0)
        {
            elementJS = elementJS.Replace("\"", "");
            sb.Append("\"" + elementJS.ToUpper() + "\": {" + Environment.NewLine);
            sb.Append("\"styles\"" + ": { " + System.Environment.NewLine);
            sb.Append(String.Format("\"{0}\":", elementJS.Replace(".", "")) + System.Environment.NewLine + "{" + System.Environment.NewLine);
            for (int valCount = 0; valCount < values.Count; valCount++)
            {
                sb.Append("\"type\": \"style\"," + System.Environment.NewLine);
                sb.Append("\"value\": \"" + values[valCount].Name + ":" + values[valCount].Term + ";\"," + System.Environment.NewLine);
                sb.Append("\"workareaBox\": \"" + values[valCount].Name + ":" + values[valCount].Term + ";\"" + System.Environment.NewLine);
                if (valCount - 1 == values.Count)
                {
                    sb.Append("}," + Environment.NewLine);
                }
                else
                {
                    sb.Append("}" + Environment.NewLine);
                }
            }

            sb.Append("}" + System.Environment.NewLine + "}" + System.Environment.NewLine + ",");
        }
        return sb.ToString();
    }
}